using System;
using System.Data.SqlServerCe;

namespace Demo1
{
    class Program
    {
        private static SqlCeConnection _connection;

        static void Main(string[] args)
        {
            // Create a connection to our super secure data store
            SqlCeEngine engine = new SqlCeEngine("Data Source='d:\\TestDB.sdf'; password='Password1'");
            _connection = new SqlCeConnection(engine.LocalConnectionString);
            _connection.Open();

            Console.WriteLine("NEXT: Insert all speakers");
            Console.ReadLine();

            // Insert a speaker
            SetupDatabase();

            Console.WriteLine("NEXT: Display all speakers");
            Console.ReadLine();

            // List all speakers
            WriteAllSpeakers();

            Console.WriteLine("NEXT: Best speaker");
            Console.ReadLine();

            // Display the best speaker
            WriteBestSpeaker();

            // Clean up our database 
            TearDownDatabase();

            // Close the connection and wait to exit
            _connection.Close();
            Console.ReadLine();

        }

        #region Setup Database Implementation

        /// <summary>
        /// Inserts all speakers into the database
        /// </summary>
        static void SetupDatabase()
        {
            InsertSpeaker("Simone Chiaretta");
            InsertSpeaker("Kevin Daly");
            InsertSpeaker("Rod Drury");
            InsertSpeaker("James Hippolite");
            InsertSpeaker("Guilherme Magalhaes");
            InsertSpeaker("Craig Pringle");
            InsertSpeaker("Tom Ton");
            InsertSpeaker("John-Daniel Trask");
            InsertSpeaker("Geoff Upham");
            InsertSpeaker("Nic Wise");
            InsertSpeaker("Jeremy Boyd");
            InsertSpeaker("Darryl Burling");
            InsertSpeaker("Mark Carroll");
            InsertSpeaker("Chris Auld");

            // Spacer for formatting
            Console.WriteLine();
        }

        #endregion

        #region Insert Speaker Implementation

        /// <summary>
        /// Inserts a speaker into the database
        /// </summary>
        /// <param name="name">The name of the speaker</param>
        static void InsertSpeaker(string name)
        {
            Console.WriteLine("Inserting {0}", name);
            SqlCeCommand command = _connection.CreateCommand();
            command.CommandText = string.Format("INSERT INTO Speakers ([SpeakerName]) VALUES ('{0}')", name);
            command.ExecuteNonQuery();
        }

        #endregion

        #region Write All Speakers Implementation

        /// <summary>
        /// Print all the speakers to the screen
        /// </summary>
        static void WriteAllSpeakers()
        {
            Console.WriteLine("All Speakers:{0}", Environment.NewLine);
            SqlCeCommand command = _connection.CreateCommand();
            command.CommandText = "SELECT SpeakerName FROM Speakers";
            SqlCeDataReader reader = command.ExecuteReader();

            while(reader.Read())
            {
                Console.WriteLine(reader.GetValue(0));
            }

            // Spacer for formatting
            Console.WriteLine();
        }

        #endregion

        #region Best Speaker Implementation

        /// <summary>
        /// Find tonights best speaker at the .net user group
        /// </summary>
        static void WriteBestSpeaker()
        {
            // Get the number of rows
            SqlCeCommand countCommand = _connection.CreateCommand();
            countCommand.CommandText = "SELECT count(*) FROM Speakers";
            int speakerCount = (Int32)(countCommand.ExecuteScalar());

            // Get a random number
            Random r = new Random(DateTime.Now.Millisecond);
            int bestSpeaker = r.Next(0, speakerCount);

            // Get all speakers
            SqlCeCommand command = _connection.CreateCommand();
            command.CommandText = "SELECT SpeakerName FROM Speakers";
            SqlCeDataReader reader = command.ExecuteReader();

            // Find our best speaker (NOTE: this is pretty inefficient, don't do this :)
            int count = 0;
            while (reader.Read())
            {
                if (count == bestSpeaker)
                {
                    Console.WriteLine(reader.GetValue(0));
                    break;
                }

                count++;
            }

            // Spacer for formatting
            Console.WriteLine();
        }

        #endregion

        #region Tear Down Implementation

        /// <summary>
        /// Delete all the data from the speakers table
        /// </summary>
        static void TearDownDatabase()
        {
            Console.WriteLine("Clearing all data!");
            SqlCeCommand command = _connection.CreateCommand();
            command.CommandText = "DELETE FROM Speakers";
            command.ExecuteNonQuery();

            // Spacer for formatting
            Console.WriteLine();
        }

        #endregion

    }
}
